<?php namespace Sitewit\WpPlugin;

use Sitewit\Exception\SW_Api_Exception;

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

class SW_Plugin
{
	private static $instance = null;

	public static function get_instance() {
		if ( ! self::$instance ) {
			self::$instance = new SW_Plugin;
		}

		return self::$instance;
	}

	/**
	 * A starting point to the plugin initialization.
	 * - Add hooks to create
	 *   + Config/Settings menu
	 *   + Print the tracking code onto the footer
	 */
	public function init_hooks() {
		// Some sort of housekeeping before doing anything else
		add_action( 'admin_init', array( $this, 'house_keeping' ) );

		// Check if plugin's schema needs update
		add_action( 'admin_init', array( $this, 'update_schema' ) );

		// Add a settings menu to settings/config page of the plugin
		add_action( 'admin_menu', function() {
			add_options_page(
				__( 'Search Engine Marketing', SW_TEXT_DOMAIN ),	// Page title
				__( 'Search Engine Marketing', SW_TEXT_DOMAIN ),	// Menu title
				'manage_options',
				SW_SETTING_PAGE,
				array( '\Sitewit\WpPlugin\SW_Plugin', 'config_page' )
			);
		} );

		// Load assets for admin setting page(s)
		if ( sw_is_setting_page() ) {
			add_action( 'admin_enqueue_scripts', array( $this, 'load_admin_assets' ) );
		}

		// Tell user to continue with setting up the plugin
		if ( sw_no_tracking_code() ) {
			add_action( 'admin_notices', function() {
				// Show notice only if the current user has "manage_options" capability
				if (current_user_can( 'manage_options' )) {
					$message = sprintf(
						wp_kses(
							__( '<strong>Search Engine Marketing plugin is almost ready.</strong> You must <a href="%s">link your SiteWit account</a> in order for it to work.', SW_TEXT_DOMAIN ),
							array(
								'strong' => array(),
								'a' => array( 'href' => array() )
							)
						), esc_url( sw_get_setting_page_link() )
					);

					echo Messages::add_warning( $message );
				}
			} );
		}

		// Add a handler to create account link via AJAX
		if ( is_admin() ) {
			add_action( 'wp_ajax_link_account', array( $this, 'ajax_link_account' ) );
			add_action( 'wp_ajax_reset_account', array( $this, 'ajax_reset_account' ) );
		}

		// Add tracking code to front-end footer if available
		if ( ( $tracking_script = sw_get_tracking_code() ) !== false ) {
			// This JS code containing a function call should be placed before the actual "sw.js" file call.
			add_action( 'wp_footer', function() use($tracking_script){
				echo
<<<"TRACKINGCODE"
<script type="text/javascript">
	var swPreRegister = function() { }
</script>
TRACKINGCODE;
				echo $tracking_script;
			} );
		}
	}

	/**
	 * Render the configuration/setting page.
	 * - The config page will include the process to smoothly sign the user(s) up with a SiteWit account.
	 * - When the account is setup properly, it shows the user with a set of icons to quickly navigate to SiteWit reports.
	 */
	public static function config_page() {
		// Get SW_Plugin instance because this function might get called from closure and not really have access to $this
		$inc = \Sitewit\WpPlugin\SW_Plugin::get_instance();

		// Include the correct view files
		if ( sw_no_tracking_code() ) {
			$current_user = wp_get_current_user();

			$args = array(
				'sw_signup_url' => SW_HOST . 'auth/newaccount-wp.aspx?'
													. 'aff=' . urlencode( strtolower( get_option( SW_OPTION_NAME_AFFILIATE_ID, '' ) ) )
													. '&u=' . urlencode( home_url() )                     // site url
													. '&n=' . urlencode( $current_user->display_name )    // name
													. '&e=' . urlencode( $current_user->user_email ),     // email
				'sw_login_url' => SW_HOST . 'plugins/wordpress/Default.aspx?login=true'
 			);

			$inc->view( 'config', $args );
		} else {
			// Has the tracking code already, so show the Settings page
			$inc->view( 'settings' );
		}
	}

	/**
	 * @param string $name Name of the view to be loaded. The file must locates in "views" folder.
	 * @param array  $data Data to be passed to the view
	 */
	public function view( $name, array $data = array() ) {
		$file = SW_PLUGIN_DIR . "views/{$name}.php";
		if ( true === file_exists( $file ) ) {
			// Include the template file
			include $file;
		}
	}

	/**
	 * Load some common assets for (only) admin page
	 */
	public function load_admin_assets() {
		wp_enqueue_style( 'sw-style', SW_PLUGIN_URL . 'assets/sw-style.css' );

		wp_enqueue_script( 'jquery-core' ); // In case there's no jQuery loaded yet
	}

	/**
	 * Perform link account upon receiving an AJAX request. CSRF protection enabled!
	 */
	public function ajax_link_account() {
		// Security check
		check_ajax_referer( 'sw-link-account-nonce', 'swAjaxNonce' );

		$api_token = sanitize_text_field( $_POST['apiToken'] );
		$user_token = sanitize_text_field( $_POST['userToken'] );

		$error = '';

		// Check if the tokens are working by making request to SiteWit API
		if ( $api_token && $user_token ) {
			try {
				$sw_api = new SW_Api( $api_token, $user_token );

				$account_info = $sw_api->get_account();

				if ( isset( $account_info->accountNumber ) ) {
					self::setup_plugin_data( $account_info->accountNumber );
				}
			} catch ( SW_Api_Exception $e ) {
				$error = sprintf( __( 'SW API call unsuccessful! %s', SW_TEXT_DOMAIN ), $e->getMessage() );
			} catch ( \Exception $e ) {
				// Think of something to log the catchable errors
				$error = sprintf( __( 'Caught exception: %s', SW_TEXT_DOMAIN ), $e->getMessage() );
			}
		}

		echo json_encode(array(
			'error' => $error
		));

		wp_die(); // For ajax to return immediately with the JSON
	}

	public function ajax_reset_account() {
		// Security check
		check_ajax_referer( 'sw-reset-account-nonce', 'swAjaxNonce' );

		// Check passed, so just remove database-stored value
		sw_plugin_db_cleanup( false );

		echo json_encode( array( 'success' => true ) );

		wp_die();
	}

	public function house_keeping() {
		// Set config information if available
		if ( file_exists( SW_PLUGIN_DIR . 'config.php' ) ) {
			$sw_affiliate_id = '';

			include SW_PLUGIN_DIR . 'config.php';

			if ( $sw_affiliate_id ) update_option( SW_OPTION_NAME_AFFILIATE_ID, $sw_affiliate_id );
		}

		// Set database schema version if there is not one yet
		if ( false === get_option( SW_OPTION_NAME_SCHEMA_VERSION ) ) {
			add_option( SW_OPTION_NAME_SCHEMA_VERSION, SW_PLUGIN_SCHEMA_VERSION );
		}
	}

	public function update_schema() {
		if ( ! is_admin() || ! defined( 'SW_PLUGIN_SCHEMA_VERSION' ) ) return;

		// Get current schema version. Default to 1. We might not have this before.
		$current_schema_version = get_option( SW_OPTION_NAME_SCHEMA_VERSION, '1' );

		if ( version_compare( $current_schema_version, SW_PLUGIN_SCHEMA_VERSION, '<' )) {
			$updater = new SW_Updater();

			for ( $v = $current_schema_version; $v < SW_PLUGIN_SCHEMA_VERSION; $v++ ) {
				// Check if the updater function exists. In any loop, if it's not, break immediately!
				$method_name = "UpdateFrom{$v}To" . ($v + 1);
				if ( ! method_exists( $updater, $method_name ) ) break;

				if ( $updater->{$method_name}() ) {
					// Update the current schema to the latest (code) version
					update_option( SW_OPTION_NAME_SCHEMA_VERSION, $v + 1 );
				}
			}
		}
	}

	private function setup_plugin_data( $account_id )
	{
		$tracking_script =
<<<"TRACKINGSCRIPT"
	<script type="text/javascript">
		var scheme = ( "https:" === document.location.protocol ) ? "https://" : "http://";
		document.write(decodeURI("%3Cscript src='" + scheme + "analytics.sitewit.com/v3/{$account_id}/sw.js' type='text/javascript'%3E%3C/script%3E"));
	</script>
TRACKINGSCRIPT;

		update_option( SW_OPTION_NAME_TRACKING_SCRIPT, $tracking_script );

		// Done. We don't need account/api token anymore.
		delete_option( SW_OPTION_NAME_API_TOKEN );
	}
}
